<?php

/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

/**
 * @author Eugene Melnik <e.melnik@luckywarepro.com>
 */
class DZF_Auth_Adapter_TwoFactor_GoogleAuthenticator implements Zend_Auth_Adapter_Interface
{
    /**
     * @var string
     */
    protected $_responseCode;
    
    /**
     * @var string
     */
    protected $_secretKey;

    /**
     * @var int
     */
    protected $_variation;

    /**
     * @param   string      $responseCode
     * @param   string      $secretKey
     * @param   int         $variation          (optional, default - 1)
     */
    public function __construct($responseCode, $secretKey, $variation = 1)
    {
        $this->_responseCode = $responseCode;
        $this->_secretKey = $secretKey;
        $this->_variation = $variation;
    }

    /**
     * @return  Zend_Auth_Result
     */
    public function authenticate()
    {
        $identity = null;
        $messages = array();

        require_once 'GoogleAuthenticator/GoogleAuthenticator.php';
        $ga = new Google\Authenticator\GoogleAuthenticator();

        if ($ga->checkCode($this->_secretKey, $this->_responseCode, $this->_variation))
        {
            $identity = array(
                'secretKey'     => $this->_secretKey,
                'responseCode'  => $this->_responseCode,
                'variation'     => $this->_variation
            );

            $code = Zend_Auth_Result::SUCCESS;
        }
        else
        {
            $code = Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID;
        }
        
        return new Zend_Auth_Result($code, $identity, $messages);
    }
}
