<?php

/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

/**
 * @author Eugene Melnik <e.melnik@luckywarepro.com>
 */
class DZF_Auth_Adapter_TwoFactor_U2F implements Zend_Auth_Adapter_Interface
{
    /**
     * @var array
     */
    protected $_requests;
    
    /**
     * @var array
     */
    protected $_registrations;

    /**
     * @var object
     */
    protected $_response;

    /**
     * @var string
     */
    protected static $_domain;

    /**
     * @param   array       $requests
     * @param   array       $registrations
     * @param   object      $response
     */
    public function __construct($requests, $registrations, $response)
    {
        $this->_requests = $requests;
        $this->_registrations = $registrations;
        $this->_response = $response;
    }

    /**
     * @param   string          $domain
     */
    public static function setDomain($domain)
    {
        self::$_domain = $domain;
    }

    /**
     * @return  string
     */
    public function getDomain()
    {
        return self::$_domain ? self::$_domain : Zend_Registry::get('secure');
    }

    /**
     * @return  Zend_Auth_Result
     */
    public function authenticate()
    {
        $identity = null;
        $messages = array();

        try
        {
            require_once 'u2flib_server/U2F.php';
            $u2f = new u2flib_server\U2F($this->getDomain());

            /**
             * $identity = array(
             *     keyHandle,
             *     publicKey,
             *     certificate,
             *     counter
             * );
             */
            $identity = (array) $u2f->doAuthenticate($this->_requests, $this->_registrations, $this->_response);

            $code = Zend_Auth_Result::SUCCESS;
        }
        catch (Exception $e)
        {
            if (Zend_Registry::isRegistered('logger'))
            {
                Zend_Registry::get('logger')->err(
                    sprintf(
                        "U2F Auth adapter exception: %s: %s\n\n%s",
                        $e->getCode(),
                        $e->getMessage(),
                        var_export($this, true)
                    ),
                    '2fa'
                );
            }

            $code = Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID;
        }

        return new Zend_Auth_Result($code, $identity, $messages);
    }
}
