<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

/**
 * @author Karsten Deubert <info@deubert.it>
 */
class DZF_Bootstrap extends Zend_Application_Bootstrap_Bootstrap
{

    /**
     * Initializes the plugin loader cache
     */
    protected function _initPluginloader()
    {
        // Do not cache plugins except in the production environment
        if (APPLICATION_ENV !== "production" || substr(APPLICATION_ENV, 0, 10) != 'production')
        {
            return;
        }

        $classFileIncCache = TEMP_PATH.DIRECTORY_SEPARATOR.'DZFPluginLoaderCache.php';
        if (file_exists($classFileIncCache)) {
            include_once $classFileIncCache;
        }
        Zend_Loader_PluginLoader::setIncludeFileCache($classFileIncCache);
    }

    /**
     * Initializes the namespaces and autoloader resources
     */
    protected function _initAutoloader()
    {

    }

    /**
     * Initializes and starts the session
     */
    protected function _initSessionStart()
    {
        if (
            !$this->hasPluginResource('session')
        ) {
            return;
        }

        // initialize resource plugin as defined in the config
        $this->bootstrap('session');

        // manually start the session because it may have the 'strict' option
        // set (doesn't allow automatic start)
        Zend_Session::start();
    }

    protected function _initCssTimestamp()
    {
        $cssOptions = $this->getOption('css');
        Zend_Registry::set('csstimestamp', $cssOptions['timestamp']);
    }

    /**
     * Sets the timezone
     * Current workaround to avoid notices when calling the 'date' function
     */
    protected function _initTimezone()
    {
        date_default_timezone_set('Europe/Berlin');

        //setlocale(LC_ALL, 'de_DE');
        //setlocale(LC_NUMERIC, 'en_US'); // let's overwrite the decimal separator
        //setlocale(LC_TIME, 'de_DE');
    }

    /**
     * Initializes the translations used in the application.
     * At the moment there are translations for the following resources:
     *     - Zend_Validate_Abstract
     *     - Zend_Controller_Router_Route
     *     - global (via Zend_Translate in the registry)
     *
     * They are managed in the DZF_Language class that is responsible to set
     * the currently active language.
     *
     * In addition all translators receive a separate log where the missing
     * translations will be logged.
     */
    protected function _initTranslation()
    {
        $this->bootstrap('logger');

        if (APPLICATION_ENV == "production" && $this->hasPluginResource('cachemanager'))
        {
            $this->bootstrap('cachemanager');
            $cacheManager = $this->getResource('cachemanager');
            Zend_Registry::set('cachemanager', $cacheManager);

            /*if ($cacheManager->hasCache('translate')) {
                $cache = $cacheManager->getCache('translate');
                Zend_Translate::setCache($cache);
            }*/
        }

        $formatter = new Zend_Log_Formatter_Simple('%message%' . PHP_EOL);
        $logMessageFormat = '"%locale%";"%message%"';

        $this->bootstrap('sessionStart');
        $language = DZF_Language::getInstance();

        // init application translation
        if ($this->hasPluginResource('translate'))
        {
            $this->bootstrap('translate');
            $applicationTranslator = $this->getResource('translate');
            if (APPLICATION_ENV !== 'production')
            {
                $applicationWriter = new Zend_Log_Writer_Stream(
                    LOG_PATH.DIRECTORY_SEPARATOR.APPLICATION_NAME.'.dzf.translation.application.csv');
                $applicationWriter->setFormatter($formatter);
                $applicationTranslator->setOptions(
                    array(
                        'log'             => new Zend_Log($applicationWriter),
                        'logUntranslated' => true,
                        'logMessage'      => $logMessageFormat,
                    )
                );
            }
            $language->addTranslator($applicationTranslator);
        }

        // init translators with currently active language
        $language->initTranslators();

        //var_dump($language);

        //$lang = $language->getLanguage();
        // the default locale must be set, otherwise the translation won't
        // work properly
        //Zend_Controller_Router_Route::setDefaultLocale($lang);
    }

    /**
     * Initializes the zfdebug toolbar and its according plugins.
     * The toolbar is only added in the development environment or if an
     * admin user is logged in.
     */
    protected function _initZFDebug()
    {
        $options = array();
        $options['jquery_path'] = '/js/jquery/jquery-1.3.2.min.js';
        $options['plugins'] = array(
            'Variables',
            'File' => array('base_path' => APPLICATION_PATH),
            'Memory',
            'Time',
            'Html',
            'Registry',
            'Exception'
        );

        /*
         * Check the identity and environment if ZFDebug should be initialized
         * This is done after the db has been initialized so Zend_Auth has
         * database access to retrieve the identity
         */
        $this->bootstrap('sessionStart');

        // doctrine integration
        /*if ($this->hasPluginResource('doctrine'))
        {
            $this->bootstrap('doctrine');*/
            $this->_initDoctrine();
            $options['plugins'][] = 'DZF_ZFDebug_Controller_Plugin_Debug_Plugin_Doctrine';
        //}

        $hideTools = true;

        if (substr(APPLICATION_ENV, 0, 11) === 'development')
        {
            $hideTools = false;
        }

        if ($hideTools)
        {
            return;
        }

        $autoloader = Zend_Loader_Autoloader::getInstance();
        $autoloader->registerNamespace('ZFDebug');

        # Setup the cache plugin
        if ($this->hasPluginResource('cachemanager'))
        {
            $this->bootstrap('cachemanager');
            $cacheManager = $this->getResource('cachemanager');
            Zend_Registry::set('cachemanager', $cacheManager);
            $caches = $cacheManager->getCaches();

            foreach ($caches as $name => $cache)
            {
                $caches[$name] = $cache->getBackend();
            }
            $options['plugins']['Cache']['backend'] = $caches;
        }

        $debug = new ZFDebug_Controller_Plugin_Debug($options);
        $debug->registerPlugin(new DZF_ZFDebug_Controller_Plugin_Debug_Plugin_Session());

        if ($this->hasPluginResource('frontController'))
        {
            $this->bootstrap('frontController');
            $frontController = $this->getResource('frontController');
            $frontController->registerPlugin($debug);
        }

        return $debug;
    }

    /**
     * Initializes the logger.
     * Gets the logger defined in the application.ini and adds a specific
     * log writer
     * for the zfdebug toolbar.
     * There is also an additional log priority for the zfdebug logger added.
     */
    protected function _initLogger()
    {
        $this->bootstrap('timezone');
        if ($this->hasPluginResource('log'))
        {
            $this->bootstrap('log');
            $log = $this->getResource('log');
        }
        else
        {
            $log = new Zend_Log();
        }
        // Add a new log priority which writes only into ZFDebug and not into
        // the log file (this is defined in the application.ini)
        $log->addPriority('zfdebug', 8);
        $this->bootstrap('ZFDebug');
        if ($this->hasResource('ZFDebug'))
        {
            $plugin = $this->getResource('ZFDebug');
            $zfDebugWriter = new DZF_Log_Writer_ZFDebug($plugin);
            $log->addWriter($zfDebugWriter);
        }
        Zend_Registry::set('logger', $log);
    }

    protected function _initPagination()
    {
        Zend_Paginator::setDefaultItemCountPerPage(10);
    }

    protected function _initDoctrine()
    {
        $this->bootstrap('autoloader');

        $doctrineConfig = $this->getOption('doctrine');
        $dbConfig = $doctrineConfig['db'];

        if (!isset($dbConfig['dsn']))
        {
            return;
        }

        if (strpos(APPLICATION_ENV, 'testing') === 0 && getenv('BUILD_BRANCH') && strlen(getenv('BUILD_BRANCH')))
        {
            $buildBranchName = getenv('BUILD_BRANCH');
            $overwriteFilePath = APPLICATION_PATH . '/configs/build-branch-'.$buildBranchName.'.ini';
            if (file_exists($overwriteFilePath))
            {
                $iniConfigArray = parse_ini_file($overwriteFilePath);
                $dbConfig = array_merge($dbConfig, $iniConfigArray);
            }
        }

        require_once 'Doctrine.php';

        $autoloader = Zend_Loader_Autoloader::getInstance();
        $autoloader->pushAutoloader(array('Doctrine', 'autoload'), 'Doctrine');
        $autoloader->pushAutoloader(array('Doctrine', 'modelsAutoload'));

        $manager = Doctrine_Manager::getInstance();
        $manager->setCharset('utf8');
        $manager->setCollate('utf8_general_ci');
        $manager->setAttribute(Doctrine::ATTR_MODEL_LOADING, Doctrine::MODEL_LOADING_CONSERVATIVE);
        $manager->setAttribute(Doctrine::ATTR_QUOTE_IDENTIFIER, true);

        if (!defined('DOCTRINE_SKIP_CONNECTION') || !DOCTRINE_SKIP_CONNECTION)
        {
            // default connection
            $adapter = array($dbConfig['dsn'], $dbConfig['username'], $dbConfig['password']);
            $connection = $manager->connection($adapter, 'default');
            $connection->setCharset('UTF8');
            $connection->setAttribute(Doctrine_Core::ATTR_USE_NATIVE_ENUM, true);
        }

        Doctrine::loadModels($doctrineConfig['models_path']);
    }

    /**
     * Adds domain from application.ini to registry to support full links from everywhere
     *
     * @return void
     */
    protected function _initDomain()
    {
        $domainOptions = $this->getOption('domain');
        Zend_Registry::set('domain', $domainOptions['domain']);
        Zend_Registry::set('domain.static', $domainOptions['static']);
        Zend_Registry::set('secure', $domainOptions['secure']);
        Zend_Registry::set('cookiedomain', $domainOptions['cookiedomain']);
    }

    /**
     * Adds debugging config from application.ini to registry
     *
     * @return void
     */
    protected function _initDebugging()
    {
        $debuggingOptions = $this->getOption('debugging');
        Zend_Registry::set('debugging.sendmails', $debuggingOptions['sendmails']);
        Zend_Registry::set('debugging.emailtarget', $debuggingOptions['emailtarget']);
    }

    /**
     * Reads version info from resources/version.php - filled via deployment on production and beta environments
     *
     * @return void
     */
    protected function _initVersionInfo()
    {
        $versionInfo = APPLICATION_ENV;
        if (APPLICATION_ENV == 'production' || APPLICATION_ENV == 'staging')
        {
            $versionDataPath = APPLICATION_PATH.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'resources'.DIRECTORY_SEPARATOR.'version.php';
            if (file_exists($versionDataPath))
            {
                $versionData = require_once $versionDataPath;
            }
            $versionInfo .= ' #'.$versionData['number'].' id: '.$versionData['id'].' rev: '.$versionData['svn_revision'];
        }
        Zend_Registry::set('versionInfo', $versionInfo);
    }

    /**
     * @return void
     */
    protected function _initUserIp()
    {
        $userIp = '127.0.0.1';
        if ($_SERVER && is_array($_SERVER) && array_key_exists('REMOTE_ADDR', $_SERVER))
        {
            $userIp = $_SERVER['REMOTE_ADDR'];
            if (array_key_exists('HTTP_X_FORWARDED_FOR', $_SERVER))
            {
                if (strpos($_SERVER['HTTP_X_FORWARDED_FOR'], ', ') !== false)
                {
                    $forwardedIps = explode(', ', $_SERVER['HTTP_X_FORWARDED_FOR']);
                    $userIp = $forwardedIps[0];
                }
                else
                {
                    $userIp = $_SERVER['HTTP_X_FORWARDED_FOR'];
                }
            }
        }

        Zend_Registry::set('user.ip', $userIp);
    }
}