<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

/**
 * @author Karsten Deubert <info@deubert.it>
 */
class DZF_Controller_Plugin_Annotation_Https extends DZF_Controller_Plugin_Annotation_Abstract
{
    public function preDispatch(Zend_Controller_Request_Abstract $request)
    {
        if (Zend_Registry::get('secure') == Zend_Registry::get('domain'))
        {
            return;
        }
        
        /** @var $request Zend_Controller_Request_Http */
        if (!$request instanceof Zend_Controller_Request_Http)
        {
            return;
        }

        /** @var $zfDebugTimePlugin ZFDebug_Controller_Plugin_Debug_Plugin_Time */
        $zfDebugTimePlugin = Zend_Controller_Front::getInstance()->getPlugin('ZFDebug_Controller_Plugin_Debug_Plugin_Time');
        if ($zfDebugTimePlugin && is_object($zfDebugTimePlugin))
        {
            $zfDebugTimePlugin->mark('annotation_https start');
        }

        $docBlock = $this->_getDocblockForControllerAction();

        $isSecureRoute = false;

        $pattern = '#@https (true|false|skip)#iu';

        $matches = array();
        $matchesCount = preg_match($pattern, $docBlock, $matches);

        if ($matchesCount)
        {
            $isSecureRoute = $matches[1] === 'true';
            if ($matches[1] === 'skip')
            {
                if ($zfDebugTimePlugin && is_object($zfDebugTimePlugin))
                {
                    $zfDebugTimePlugin->mark('annotation_https stop (match - skip)');
                }
                return;
            }
        }

        /** @var $request Zend_Controller_Request_Http */
        $request = $this->getRequest();

        if ($this->getResponse()->isException())
        {
            return;
        }

        //no https
        if ($request->getServer('HTTPS', null) !== 'on')
        {
            //but route should use https -> redirect
            if ($isSecureRoute && !$request->isXmlHttpRequest())
            {
                $this->getResponse()->setHttpResponseCode(301);
                $this->getResponse()->clearAllHeaders();
                $this->getResponse()->setHeader('Location', Zend_Registry::get('secure') . $request->getServer('REQUEST_URI'), true);
                //$this->getResponse()->setHeader('Connection', 'close'); //bug in IE?
                $request->setDispatched();
            }
        }
        else //it is HTTPS
        {
            //but the route shouldn't use https -> redirect
            if (!$isSecureRoute && !$request->isXmlHttpRequest())
            {
                $this->getResponse()->setHttpResponseCode(301);
                $this->getResponse()->clearAllHeaders();
                $this->getResponse()->setHeader('Location', Zend_Registry::get('domain') . $request->getServer('REQUEST_URI'), true);
                //$this->getResponse()->setHeader('Connection', 'close'); //bug in IE?
                $request->setDispatched();
            }
        }

        if ($zfDebugTimePlugin && is_object($zfDebugTimePlugin))
        {
            $zfDebugTimePlugin->mark('annotation_https stop (match)');
        }
    }
}