<?php

/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

/**
 * @author Eugene Melnik <e.melnik@luckywarepro.com>
 */
class DZF_Log_Writer_Doctrine extends Zend_Log_Writer_Abstract
{
    protected $_defaultNamespace = 'core';

    protected $_isBufferedMode = false;
    protected $_logsBuffer;

    /**
     * @param string $defaultNamespace
     * @param bool $bufferedMode
     */
    public function __construct($defaultNamespace = null, $bufferedMode = false)
    {
        if ($defaultNamespace)
        {
            $this->_defaultNamespace = $defaultNamespace;
        }

        $this->_isBufferedMode = $bufferedMode;
        $this->_logsBuffer = new Doctrine_Collection('DZF_Log_Writer_Doctrine_Model');
    }

    /**
     * @param string $namespace
     */
    public function setNamespace($namespace)
    {
        $this->_defaultNamespace = $namespace;
    }

    /**
     * @param bool $bufferedMode
     */
    public function setBufferedMode($bufferedMode = true)
    {
        $this->_isBufferedMode = $bufferedMode;

        if (!$bufferedMode)
        {
            $this->flush();
        }
    }

    /**
     * Perform shutdown activites such as closing open resources
     */
    public function shutdown()
    {
        $this->flush();
    }

    /**
     * Write a message to the log.
     *
     *  array_keys($event) = array(
     *     "timestamp",
     *     "message",
     *     "priority",
     *     "priorityName",
     *     "info"
     *   );
     *
     * @param array $event
     */
    protected function _write($event)
    {
        $logRecord = new DZF_Log_Writer_Doctrine_Model();
        $logRecord->set('timestamp', date('Y-m-d H:i:s', strtotime($event['timestamp'])));
        $logRecord->set('priority', $event['priority']);
        $logRecord->set('message', $event['message']);

        $info = isset($event['info']) ? $event['info'] : $this->_defaultNamespace;
        $logRecord->set('namespace', $info);

        if ($this->_isBufferedMode)
        {
            $this->_logsBuffer->add($logRecord);
        }
        else
        {
            $logRecord->save();
        }
    }

    public function flush()
    {
        if ($this->_logsBuffer->count() > 0)
        {
            $this->_logsBuffer->save();
        }
    }

    /**
     * Construct a Zend_Log driver
     *
     * @param  array|Zend_Config $config
     * @return Zend_Log_FactoryInterface
     */
    public static function factory($config)
    {
        throw new Zend_Exception('DZF_Log_Writer_Doctrine does not currently implement a factory');
    }
}
