<?php

/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

/**
 * @author Eugene Melnik <e.melnik@luckywarepro.com>
 */
class DZF_Log_Writer_Doctrine_Model extends Doctrine_Record
{
    const LOGS_TABLE_NAME = 'dzf_logs';

    public function setTableDefinition()
    {
        $this->setTableName(self::LOGS_TABLE_NAME);

        $this->hasColumn('id', 'integer', 10, array(
            'type' => 'integer',
            'unsigned' => true,
            'primary' => true,
            'autoincrement' => true
        ));

        $this->hasColumn('timestamp', 'timestamp', null, array(
            'notnull' => true
        ));

        $this->hasColumn('priority', 'int', 4, array(
            'notnull' => true
        ));

        $this->hasColumn('namespace', 'string', 100, array(
            'notnull' => true
        ));

        $this->hasColumn('message', 'text', null, array(
            'notnull' => true
        ));

        $this->index('namespace', array('fields' => 'namespace'));
    }

    public function delete(Doctrine_Connection $conn = null)
    {
        throw new DZF_Exception('Deleting of the log records is prohibited!');
    }

    public static function getPriorityStringMapping()
    {
        return array(
            Zend_Log::EMERG => 'Emergency',
            Zend_Log::ALERT => 'Alert',
            Zend_Log::CRIT  => 'Critical',
            Zend_Log::ERR   => 'Error',
            Zend_Log::WARN  => 'Warning',
            Zend_Log::NOTICE => 'Notice',
            Zend_Log::INFO  => 'Informational',
            Zend_Log::DEBUG => 'Debug'
        );
    }

    public function getPriorityString()
    {
        $priority = $this->get('priority');
        $map = self::getPriorityStringMapping();

        if (isset($map[$priority]))
        {
            return $map[$priority];
        }
        else
        {
            return $priority;
        }
    }

    public function getPriorityLabelClass()
    {
        $priority = $this->get('priority');

        $map = array(
            Zend_Log::EMERG => 'label label-danger',
            Zend_Log::ALERT => 'label label-danger',
            Zend_Log::CRIT  => 'label label-danger',
            Zend_Log::ERR   => 'label label-danger',
            Zend_Log::WARN  => 'label label-warning',
            Zend_Log::NOTICE => 'label label-info',
            Zend_Log::INFO  => 'label label-success',
            Zend_Log::DEBUG => 'label label-default'
        );

        if (isset($map[$priority]))
        {
            return $map[$priority];
        }
        else
        {
            return '';
        }
    }

    /**
     * Returns the record representation as an array
     *
     * @param boolean $deep         whether to include relations
     * @param boolean $prefixKey    not used
     * @return array
     */
    public function toArray($deep = true, $prefixKey = false)
    {
        $result = parent::toArray($deep, $prefixKey);

        if (is_array($result))
        {
            $result['priority_string'] = $this->getPriorityString();
            $result['priority_label_class'] = $this->getPriorityLabelClass();
        }

        return $result;
    }
}
