<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

/**
 * @author Karsten Deubert <info@deubert.it>
 */
class DZF_Log_Writer_ZFDebug extends Zend_Log_Writer_Abstract
{
    const TAB_NAME_PREFIX = 'Log';
    const TAB_NAME_DEFAULT = 'Default';

    protected $_tabNameDefault;

    protected $_tabNamePrefix;

    protected $_plugin;

    protected $_tabs;

    public function __construct(
        $plugin,
        $tabNameDefault = self::TAB_NAME_DEFAULT,
        $tabNamePrefix = self::TAB_NAME_PREFIX
    )
    {
        if (!$plugin instanceof ZFDebug_Controller_Plugin_Debug) {
            throw new Zend_Log_Exception(
                'Plugin must be an instance of ZFDebug_Controller_Plugin_Debug'
            );
        }
        $this->_plugin = $plugin;
        $this->_tabNameDefault = $tabNameDefault;
        $this->_tabNamePrefix = $tabNamePrefix;
        $this->_tabs = array();
        $this->_formatter = new DZF_Log_Formatter_Dump();
    }

    protected function _write($event)
    {
        $tabName = $this->_tabNameDefault;
        if (isset($event['tabName'])) {
            $tabName = $event['tabName'];
        } else if (isset($event['info'])) {
            /**
             * 'info' is the default array key for additional params,
             * if 'tabName' is not specified take this value instead.
             */
            if (is_array($event['info'])) {
                $tabName = reset($event['info']);
                $tabName = current($event['info']);
            } else if (is_string($event['info'])) {
                $tabName = $event['info'];
            }
        }
        if (empty($tabName)) {
            $tabName = self::TAB_NAME_DEFAULT;
        }

        $textPlugin = null;
        if (array_key_exists($tabName, $this->_tabs)) {
            $textPlugin = $this->_tabs[$tabName];
        } else {
            $textPlugin = new ZFDebug_Controller_Plugin_Debug_Plugin_Text();
            $textPlugin->setIdentifier($tabName);
            $this->_tabs[$tabName] = $textPlugin;
            if (!empty($this->_tabNamePrefix)) {
                $tabName = $this->_tabNamePrefix . ": " . $tabName;
            }
            $textPlugin->setTab($tabName);
            $this->_plugin->registerPlugin($textPlugin);
        }

        $panel = $this->_formatter->format($event);

        $textPlugin->setPanel($textPlugin->getPanel().$panel);
    }

    static public function factory($config)
    {
        throw new Zend_Exception(
            'DZF_Log_Writer_ZFDebug does not currently implement a factory'
        );
    }
}