<?php

/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

/**
 * @author Rostislav Boiko <r.boyko@luckywarepro.com>
 */
abstract class DZF_Migration_Versionize extends Doctrine_Migration_Base {

    protected $_modelName;

    protected function _getTableName() {
        $model = $this->_modelName;
        return Doctrine::getTable($model)->getTableName();
    }

    protected $_v_table_name;

    protected function _getVersionTableName() {
        if (!isset($this->_v_table_name)) {
            $this->_v_table_name = $this->_getAuditPlugin()->getTable()->getTableName();
        }
        return $this->_v_table_name;
    }

    protected function _getSqlForVersions() {
        
    }

    protected $_audit_plugin;

    protected function _getAuditPlugin() {
        if (!isset($this->_audit_plugin)) {
            $model = $this->_modelName;
            $export = new Doctrine_Export();
            $audit = null;
            foreach ($export->getAllGenerators(Doctrine::getTable($model)) as $pugin) {//search for AutitLogPlugin
                if ($pugin instanceof Doctrine_AuditLog) {//this is it!
                    $audit = $pugin;
                    break;
                }
            }
            if (!$audit) {
                throw new DZF_Exception('Audit plugin is not active!');
            }

            $this->_audit_plugin = $audit;
        }
        return $this->_audit_plugin;
    }

    public function up() {
        if (empty($this->_modelName)) {
            throw new Exception('Model name is not specified');
        }
        $this->_getVersionTableName();
        $this->addColumn($this->_getTableName(), 'version', 'int', 8, array(
            'notnull' => true,
            'default' => 0
        ));
        $sql = Doctrine_Core::generateSqlFromArray(array($this->_modelName));
        $conn = Doctrine_Manager::getInstance()->connection();
        foreach ($sql as $q) {
            if (false !== strpos($q, $this->_getVersionTableName())) {//if sql related to versionizing
                $conn->execute($q);
            }
        }
    }

    public function down() {
        if (empty($this->_modelName)) {
            throw new Exception('Model name is not specified');
        }
        $this->_getVersionTableName();
        $this->removeColumn($this->_getTableName(), 'version');
        $this->dropTable($this->_getVersionTableName());
    }

}
